@@ -336,6 +336,8 @@ impl Visit for SpanVisualizer<'_> {
 
     mtd!(SimpleBlock, visit_simple_block);
 
+    mtd!(ComponentValue, visit_component_value);
+
     mtd!(Function, visit_function);
 
     mtd!(Color, visit_color);
@@ -400,8 +402,6 @@ impl Visit for SpanVisualizer<'_> {
 
     mtd!(UnicodeRange, visit_unicode_range);
 
-    mtd!(Value, visit_value);
-
     mtd!(CalcSum, visit_calc_sum);
 
     mtd!(CalcProductOrOperator, visit_calc_product_or_operator);
